# Implementation of the SCORE method
# (1) EVD of adjacency matrix
# (3) arrange eigenvectors
# (4) normalize spec emb
# (5) KMeans clustering

from .meth_utils import *
from scipy import linalg
from scipy.sparse import csr_matrix
from scipy.sparse.linalg import eigsh
#from sklearn.preprocessing import normalize
from sklearn.cluster import KMeans

def SCORE(edges, num_nodes, num_clus, seed=None):
    '''
    Function to implement SCORE alg (via dense EVD)
    :param edges: edge list
    :param num_nodes: number of nodes
    :param num_clus: number of clusters (i.e., K)
    :param seed: random seed
    :return: clustering result
    '''
    # ====================
    adj = get_adj(edges, num_nodes)
    # ==========
    eigvals_, eigvecs_ = linalg.eig(adj)
    eigvals = np.array([eigvals_[i].real for i in range(num_nodes)])
    sort_idx = sorted(range(len(eigvals)), key=lambda k: eigvals[k], reverse=True)
    #eigvals = eigvals[sort_idx]
    # ==========
    eigvecs = eigvecs_.real[:, sort_idx]
    spec_emb = eigvecs[:, 1:num_clus]
    # ==========
    for r in range(num_clus-1):
        spec_emb[:, r] /= eigvecs[:, 0]
    # ==========
    kmeans = KMeans(n_clusters=num_clus, random_state=seed).fit(spec_emb)
    clus_res = kmeans.labels_

    return clus_res

def SCORE_sp(edges, num_nodes, num_clus, seed=None):
    '''
    Function to implement SCORE alg (via sparse EVD)
    :param edges: edge list
    :param num_nodes: number of nodes
    :param num_clus: number of clusters (i.e., K)
    :param seed: random seed
    :return: clustering result
    '''
    # ====================
    src_idxs, dst_idxs, vals = get_adj_sp(edges)
    adj_sp = csr_matrix((vals, (src_idxs, dst_idxs)), shape=(num_nodes, num_nodes))
    # ==========
    K = min(num_clus+200, num_nodes)
    eigvals_, eigvecs_ = eigsh(adj_sp, k=K)
    eigvals = np.array(eigvals_)
    sort_idx = sorted(range(len(eigvals)), key=lambda k: eigvals[k], reverse=True)
    # eigvals = eigvals[sort_idx]
    # ==========
    eigvecs = eigvecs_.real[:, sort_idx]
    spec_emb = eigvecs[:, 1:num_clus]
    # ==========
    for r in range(num_clus-1):
        spec_emb[:, r] /= eigvecs[:, 0]
    # ==========
    kmeans = KMeans(n_clusters=num_clus, random_state=seed).fit(spec_emb)
    clus_res = kmeans.labels_

    return clus_res
